@@ -28,9 +28,11 @@
 #include<QGraphicsItemGroup>
 #include "screenlitkeeper.h"
 #include "level.h"
+#include "levelset.h"
 #include <QAction>
 #include <QTimer>
 #include <QSystemDeviceInfo>
+#include <QTime>
 
 using namespace QtMobility;
 
@@ -86,10 +88,6 @@ public slots:
 
     void softContinue();
 
-    void createAboutBoxItems();
-
-    void createVictoryItems();
-
     void setItemPointersNull();
 
     void turnPauseOn();
@@ -100,6 +98,8 @@ public slots:
 
 
 
+
+
 protected:
 
     /*! Gives a pointer to a random position if a free one is found. Otherwise returns NULL.
@@ -110,6 +110,10 @@ protected:
 
     void createMenuItems();
     void prepareForMenu(QGraphicsItem * pItem);
+    void createAboutBoxItems();
+    void createVictoryItems();
+    void createLevelCompletedItems();
+    void createSelectLevelsetFromListItems();
 
     const QString ghostImageFilename_;
     const QString rockImageFilename_;
@@ -136,11 +140,21 @@ protected:
     QGraphicsTextItem * pAboutItem_;
     QGraphicsTextItem * pQuitItem_;
     QGraphicsTextItem * pMinimizeItem_;
+    QGraphicsTextItem * pChooseLevelsetItem_;
 
     QGraphicsTextItem * pVictoryCongratulationsItem_;
+    QGraphicsTextItem * pVictoryScoreItem_;
     QGraphicsTextItem * pAboutBoxItem_;
 
-    QList<Level> levelList_;
+    QGraphicsTextItem * pLevelCompletedItem_;
+
+    QGraphicsTextItem * pSelectLevelsetFromListItem_;
+
+    QList<QGraphicsItem *>  levelsetItems_;
+
+    QList<Levelset> availableLevelsets_;
+
+    Levelset levelset_;
 
     int currentLevel_;
 
@@ -157,6 +171,10 @@ protected:
 
     QTimer deviceLockPollTimer_;
 
+    QTime scoreCounter_;
+    int totalScore_;
+    int levelScore_;
+
 };
 
 #endif // SEASCENE_H
